<?php
/* 
   Master Form .PHP
   Independent Copy and Delete Module
   (Requires muti-byte functionality and MySQLi functionality)
   Version 1.0
   May 30, 2018

   Will Bontrager
   https://www.willmaster.com/
   Copyright 2018 Will Bontrager Software LLC

   Use of this software is subject
   to the Will Bontrager Software, LLC
   Multi-Domain Software License Agreement
   https://www.willmaster.com/software/MultiDomainLicenseAgreement.html
*/

/* *** Customization *** */

// Specify how long to keep the latest file submission records. 
//   Use a number followed by the letter h, d, m, or y (letter 
//   representing hours, days, months, or years, respectively). 
//   An example is '3m'. Alternatively what the letter 
//   represents can be written out, like '3months'.

$HowLongToKeepLastestRecords = '1d';

// Specify the location and name of the file where downloads 
//   are to be written. Make the location relative to document 
//   root. The file needs to be in a protected directory. The 
//   file name extension must be .csv or .json, which is how 
//   this software determines the format for saving the data. 
//   The file name you specify here will be prepended with 
//   the table name so table data won't get mixed up with 
//   each other.

$LocationForDownloadedFile = '/mfphp/downloaded/MFdownload.json';

// Specify 'yes' if every file save shall be in a separate file. 
//   If so specified, the current date and time (UTC) will be 
//   prepended to the file name (after the table name is prepended). 
//   If not specified with 'yes', the file save will be appended 
//   to the current downloaded file for that table.

$KeepEveryFileSaveSeparate = 'yes';

/* *** End of Customization *** */

mb_internal_encoding('UTF-8');
ini_set('display_errors', '1');
class MySQLclass
{
	private $MySQLi;
	private $result;
	public $numRowsInResult;
	public $lastID;
	public $error;

	function __construct()
	{
		$this->numRowsInResult = 0;
		$this->lastID = 0;
		$this->error = array();
		$th = func_get_args();
		if( count($th) )
		{
			foreach( $th[0] as $k => $v ) { $this->$k = $v; }
		}
		$this->MySQLi = new mysqli( $this->hostname, $this->username, $this->password, $this->database );
		if( $this->MySQLi->connect_errno )
		{
			$this->error[] = 'Unable to connect to the database server (error number ' . $this->MySQLi->connect_errno . '):<br> ' . $this->MySQLi->connect_error;
		}
		if( ! $this->MySQLi->set_charset('utf8') )
		{
			$this->error[] = $this->AppendErrorNotationToErrorMessage('Error loading character set utf8');
		}
		return $this->error;
	} # function __construct()

	function __destruct()
	{
		$this->MySQLi->close();
	} # function __destruct()

	public function SetMySQLSessionTimezone($tz)
	{
		$sql = "SET time_zone = '$tz'";
		if( ! @mysqli_query($this->MySQLi,$sql) )
		{
			$this->error[] = $this->AppendErrorNotationToErrorMessage("Unable to set timezone $tz for MySQL.");
		}
	} # public function SetMySQLSessionTimezone()

	public function DeleteRecords($table,$where,$limit='')
	{
		$this->error = array();
		$sql = "DELETE FROM $table WHERE $where ";
		if( $limit ) { $sql .= "LIMIT $limit"; }
		if( ($this->result = @mysqli_query($this->MySQLi,$sql)) ) { return true; }
		$this->AppendErrorNotationToErrorMessage('Records deletion failed.');
		return false;
	} # public function DeleteRecords()

	public function SelectRecordData($arr)
	{
		$this->error = array();
		if( empty($arr['tablename']) )
		{
			$this->AppendErrorNotationToErrorMessage('Table name is required.');
			return false;
		}
		$sql = 'SELECT ';
		if( isset($arr['cols']) )
		{
			if( is_array($arr['cols']) ) { $sql .= implode(',',$arr['cols']) . ' '; }
			else { $sql .= implode(',',preg_split('/[, ]+/',$arr['cols'])) . ' '; }
		}
		else { $sql .= '* '; }
		$sql .= "FROM {$arr['tablename']} ";
		if( isset($arr['where']) ) { $sql .= "WHERE {$arr['where']} "; }
		if( isset($arr['order']) ) { $sql .= "ORDER BY {$arr['order']} "; }
		$limit = isset($arr['limit']) ? preg_replace('/[^\d\,]+/','',$arr['limit']) : 0;
		$offset = isset($arr['offset']) ? intval($arr['offset']) : 0;
		if( $offset and (! $limit ) ) { $limit = 9999999; }
		if( $limit ) { $sql .= "LIMIT $limit "; }
		if( $offset ) { $sql .= "OFFSET $offset "; }
		if( ($this->result = @mysqli_query($this->MySQLi,$sql)) )
		{
			$this->numRowsInResult = mysqli_num_rows($this->result);
			return true;
		}
		$this->AppendErrorNotationToErrorMessage('Selection failed.');
		return false;
	} # public function SelectRecordData()

	public function GetRow()
	{
		if( ($row = mysqli_fetch_assoc($this->result)) ) { return $row; }
		return array();
	} # public function GetRow()

	public function FreeResult()
	{
		$this->result->close();
	} # public function FreeResult()

	private function AppendErrorNotationToErrorMessage($s)
	{
		return rtrim($s,":. \t\n\r\0\x0B") . ' (error number ' . $this->MySQLi->errno . '): ' . $this->MySQLi->error;
	} # private function AppendErrorNotationToErrorMessage()
} # class MySQLclass
date_default_timezone_set('UTC');
$InstallationDirectory = __DIR__;
include_once("$InstallationDirectory/MasterForm_config.php");
$ControlTable = $MySQL_table_prefix . 'mfphp_control';
$MySQL = new MySQLclass( array( 'hostname'=>$MySQL_hostname, 'username'=>$MySQL_username, 'password'=>$MySQL_password, 'database'=>$MySQL_database ) );
if( count($MySQL->error) ) { FatalError(); }
$MySQL->SetMySQLSessionTimezone('+00:00');

// Get list of tables. Seed array with 'default_form_table'.)
$TablesList = array('default_form_table');
$success = $MySQL->SelectRecordData( 
	array(
		'cols' => array( 'table_name' ), 
		'tablename' => $ControlTable, 
		)
	);
if( $success )
{
	while( $row = $MySQL->GetRow() ) { $TablesList[] = $row['table_name']; }
	$MySQL->FreeResult();
}

// Do setup, then process each table.
$FNlead = ( trim(substr(strtolower($KeepEveryFileSaveSeparate),0,1)) == 'y' ) ? date('Y-m-d_H-i-s_') : '';
$ta = explode('/',$LocationForDownloadedFile);
$FileName = array_pop($ta);
$LocationForDownloadedFile = $_SERVER['DOCUMENT_ROOT'] . implode('/',$ta);
$ta = explode('.',$FileName);
$format = array_pop($ta);
$format = ( trim(strtolower($format)) == 'csv' ) ? 'CSV' : 'JSON';
$ta = array();
$Cutoff = date( 'Y-m-d H:i:s', GetCutoffTime($HowLongToKeepLastestRecords) );
foreach( $TablesList as $table )
{
	$KeyList = array();
	// Copy records before deletion.
	$f = "$LocationForDownloadedFile/$FNlead$table-$FileName";
	$success = $MySQL->SelectRecordData( 
		array(
			'cols' => array( '*' ), 
			'tablename' => "$MySQL_table_prefix$table", 
			'where' => "when_created<='$Cutoff'", 
			)
		);
	if( $success )
	{
		$csvexists = false;
		while( $row = $MySQL->GetRow() )
		{
			$KeyList[] = $row['primary_key'];
			unset($row['primary_key']);
			unset($row['when_updated']);
			if( $format == 'CSV' )
			{
				if( ! $csvexists )
				{
					$csvexists = true;
					if( ! file_exists($f) )
					{
						$arr = array();
						foreach( $row as $k => $v ) { $arr[] = $k; }
						file_put_contents($f,implode(',',$arr)."\n");
					}
				}
				file_put_contents($f,MakeCSVline($row)."\n",FILE_APPEND);
			}
			else { file_put_contents($f,json_encode($row)."\n",FILE_APPEND); }
		}
		$MySQL->FreeResult();
	}
	// Delete copied records.
	foreach( $KeyList as $key ) { $MySQL->DeleteRecords("$MySQL_table_prefix$table","primary_key=$key",1); }
}
echo 'OK';
exit;

function FatalError()
{
	global $MySQL;
	echo 'Unanticipated error.';
	echo '<pre>', print_r($MySQL->error,true), '</pre>';
	exit;
} # function FatalError()

function GetCutoffTime($v)
{
	$i = intval($v);
	$v = strtolower($v);
	preg_match('/^\d+(.*)$/',$v,$matches);
	$directive = trim($matches[1]);
	if( empty($directive) ) { return (time()-$i); }
	$directive = substr($directive,0,1);
	list($y,$m,$d,$hh,$mm,$ss) = explode(' ',date('Y n j G i s'));
	$mm = intval($mm);
	$ss = intval($ss);
	switch($directive)
	{
		case 'y' : return mktime($hh,$mm,$ss,$m,$d,($y-$i));
		case 'm' : 
			if( $d > 28 )
			{
				$endOfMonth = date('t',mktime(12,30,30,($m-$i),2,$y));
				if( $d > $endOfMonth ) { $d = $endOfMonth; }
			}
			return mktime($hh,$mm,$ss,($m-$i),$d,$y);
		case 'd' : return mktime($hh,$mm,$ss,$m,($d-$i),$y);
		default  : return (time()-$i);
	}
	return 0;
} # function GetCutoffTime()


function MakeCSVline($arr)
{
	$retval = array();
	foreach( $arr as $k => $v )
	{
		if( preg_match('/[",]/',$v) )
		{
			$v = str_replace('"',"\\".'"',$v);
			$v = '"' . $v . '"';
		}
		$retval[] = $v;
	}
	return implode(',',$retval);
} # function MakeCSVline()

?>